@@ -3,8 +3,8 @@
 #ifdef WIN32
 #include "winsock2.h"
 #endif
-#include "ccv-src/lib/3rdparty/sha1.h"
-#include "ccv-src/lib/3rdparty/sha1.c"
+#include "ccv-src/lib/3rdparty/sha1/sha1.h"
+#include "ccv-src/lib/3rdparty/sha1/sha1.c"
 #include "ccv-src/lib/ccv.h"
 #include "ccv-src/lib/ccv_basic.c"
 #include "ccv-src/lib/ccv_algebra.c"
